package com.ruoyi.project.data.price.service.impl;

import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.*;

import com.baomidou.dynamic.datasource.annotation.DS;
import com.ruoyi.common.exception.CustomException;
import com.ruoyi.common.utils.LoadUtil;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.framework.config.UVConfig;
import com.ruoyi.project.common.UVResponse;
import com.ruoyi.project.common.VueSelectModel;
import com.ruoyi.project.data.cases.domain.OriginalOfficeCase;
import com.ruoyi.project.data.cases.mapper.OriginalOfficeCaseMapper;
import com.ruoyi.project.data.cases.mapper.sync.DownloadOriginalOfficeCaseMapper;
import com.ruoyi.project.data.price.domain.OfficeBasePriceModifyModel;
import com.ruoyi.project.data.price.domain.UltimateOfficeBasePrice;
import com.ruoyi.project.data.price.mapper.UltimateOfficeBasePriceMapper;
import com.ruoyi.project.data.price.service.IUltimateOfficeBasePriceService;
import com.ruoyi.project.system.service.impl.SysUserServiceImpl;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.jdbc.core.namedparam.SqlParameterSource;
import org.springframework.jdbc.core.namedparam.SqlParameterSourceUtils;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

/**
 * 办公基价Service业务层处理
 *
 * @author ruoyi
 * @date 2020-05-20
 */
@Service
@DS("compute")
public class UltimateOfficeBasePriceServiceImpl implements IUltimateOfficeBasePriceService {

    private static final Logger log = LoggerFactory.getLogger(SysUserServiceImpl.class);

    @Autowired
    private UltimateOfficeBasePriceMapper officeBasePriceUltimateMapper;
    @Autowired
    private UVConfig uvConfig;
    @Autowired
    private JdbcTemplate jdbcTemplate;
    @Autowired
    private NamedParameterJdbcTemplate namedParameterJdbcTemplate;
    @Autowired
    private DownloadOriginalOfficeCaseMapper downloadOriginalOfficeCaseMapper;
    @Autowired
    private OriginalOfficeCaseMapper originalOfficeCaseMapper;


    private static Integer getLastYearMonth(Integer yearMonth) {
        Calendar calendar = Calendar.getInstance();
        calendar.set(yearMonth / 100, yearMonth % 100, 1);
        return new Integer(String.format("%d%02d", calendar.get(Calendar.YEAR), calendar.get(Calendar.MONTH) - 1));
    }

    /**
     * 查询办公基价列表
     *
     * @param officeBasePriceUltimate 办公基价
     * @return 办公基价
     */
    @Override
    public List<UltimateOfficeBasePrice> selectOfficeBasePriceUltimateList(UltimateOfficeBasePrice officeBasePriceUltimate) {
        officeBasePriceUltimate.setLastYearMonth(getLastYearMonth(officeBasePriceUltimate.getYearMonth()));
        return officeBasePriceUltimateMapper.getList(officeBasePriceUltimate);
    }

    @Override
    public int selectOfficeBasePriceUltimateListCount(UltimateOfficeBasePrice officeBasePriceUltimate) {
        officeBasePriceUltimate.setLastYearMonth(getLastYearMonth(officeBasePriceUltimate.getYearMonth()));
        return officeBasePriceUltimateMapper.getCount(officeBasePriceUltimate);
    }

    @Override
    public UltimateOfficeBasePrice getById(Integer yearMonth, String id) {
        Integer lastYearMonth = getLastYearMonth(yearMonth);
        return officeBasePriceUltimateMapper.getById(yearMonth, lastYearMonth, id);
    }

    @Override
    public String batchImport(Integer yearMonth, List<UltimateOfficeBasePrice> officeBasePriceUltimates,
                              String operName) {
        if (StringUtils.isNull(officeBasePriceUltimates) || officeBasePriceUltimates.size() == 0) {
            throw new CustomException("导入办公数据不能为空！");
        }
        int successNum = officeBasePriceUltimates.size();
        int failureNum = 0;
        StringBuilder successMsg = new StringBuilder();
        StringBuilder failureMsg = new StringBuilder();
        Integer lastYearMonth = getLastYearMonth(yearMonth);

        // 批量插入
        officeBasePriceUltimates.stream().parallel().forEach(inputModel -> {
            inputModel.setYearMonth(yearMonth);
            officeBasePriceUltimateMapper.insertArtificialOfficeBasePrice(inputModel);
        });

//        RestTemplate restTemplate = new RestTemplate();
//        String url = String.format(uvConfig.getAitificialOfficeBasePriceUrl(), yearMonth, lastYearMonth);
//        UVResponse<Integer> affectCount = restTemplate.getForObject(url, UVResponse.class);

        String rawSql = LoadUtil.loadContent("sql-template/update_office_price.sql");
        Calendar calendar = Calendar.getInstance();
        int year = yearMonth / 100;
        int month = yearMonth % 100;
        calendar.set(year, month, 1);
        Date firstOfMonth = calendar.getTime();
        calendar.add(Calendar.MONTH, -1);
        Date lastMonth = calendar.getTime();
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd");

        String sql = rawSql.replace("#yearMonth#", yearMonth.toString())
                .replace("#lastYearMonth#", lastYearMonth.toString())
                .replace("#today#", simpleDateFormat.format(firstOfMonth))
                .replace("#lastMonth#", simpleDateFormat.format(lastMonth));
        jdbcTemplate.update(sql);

//        String rawSql =
//        String sql = rawSql.replace("#yearMonth#", yearMonth.toString());
//        jdbcTemplate.execute(sql);

        successMsg.insert(0, "恭喜您，数据已全部导入成功！共 " + successNum + " 条");
        return successMsg.toString();
    }

    @Override
    public List<VueSelectModel> getYearMonthList() {
        return officeBasePriceUltimateMapper.getYearMonthList();
    }

    /**
     * 更新
     *
     * @param officeBasePriceUltimate
     * @return
     */
    @Override
    public int update(UltimateOfficeBasePrice officeBasePriceUltimate) {
        // 当期价格
        Integer lastYearMonth = getLastYearMonth(officeBasePriceUltimate.getYearMonth());
        UltimateOfficeBasePrice ultimateOfficeBasePrice =
                officeBasePriceUltimateMapper.getByRouteId(officeBasePriceUltimate.getYearMonth(),
                        officeBasePriceUltimate.getId());
        // 上期价格
        UltimateOfficeBasePrice lastUltimateOfficeBasePrice =
                officeBasePriceUltimateMapper.getByBuildingId(lastYearMonth,
                        officeBasePriceUltimate.getBuildingId());

        updateBasePrice(officeBasePriceUltimate, ultimateOfficeBasePrice, lastUltimateOfficeBasePrice);

        return 1;
    }

    /**
     * 基价更新
     *
     * @param inputModel
     * @param currentUltimateOfficeBasePrice
     * @param lastUltimateOfficeBasePrice
     */
    public void updateBasePrice(UltimateOfficeBasePrice inputModel,
                                UltimateOfficeBasePrice currentUltimateOfficeBasePrice,
                                UltimateOfficeBasePrice lastUltimateOfficeBasePrice) {

        OfficeBasePriceModifyModel officeBasePriceModifyModel = compareYearMonth(inputModel,
                currentUltimateOfficeBasePrice, lastUltimateOfficeBasePrice);
        if (null != officeBasePriceModifyModel) {
            officeBasePriceUltimateMapper.updateStatus(officeBasePriceModifyModel);
            officeBasePriceUltimateMapper.updateBasePriceCopyNew(officeBasePriceModifyModel);
        }

        officeBasePriceModifyModel = compareLastYearMonth(inputModel, lastUltimateOfficeBasePrice);
        if (null != officeBasePriceModifyModel) {
            officeBasePriceUltimateMapper.updateBasePriceStatus(officeBasePriceModifyModel);
            officeBasePriceUltimateMapper.updateBasePriceCopyNew(officeBasePriceModifyModel);
        }
    }

    /**
     * 上期价格
     *
     * @param inputModel
     * @param ultimateOfficeBasePrice
     * @return
     */
    private OfficeBasePriceModifyModel compareYearMonth(UltimateOfficeBasePrice inputModel,
                                                        UltimateOfficeBasePrice ultimateOfficeBasePrice,
                                                        UltimateOfficeBasePrice lastUltimateOfficeBasePrice) {
        BigDecimal inputMainPrice = inputModel.getMainPrice();
        BigDecimal mainPrice = ultimateOfficeBasePrice.getMainPrice();
        BigDecimal inputMainPriceRent = inputModel.getMainPriceRent();
        BigDecimal mainPriceRent = ultimateOfficeBasePrice.getMainPriceRent();
        if (Objects.equals(inputMainPrice, mainPrice) && Objects.equals(inputMainPriceRent, mainPriceRent)) {
            return null;
        }

        OfficeBasePriceModifyModel officeBasePriceModifyModel =
                new OfficeBasePriceModifyModel(inputModel.getId(),
                        inputModel.getYearMonth());
        officeBasePriceModifyModel.setMainPrice(inputModel.getMainPrice_1());
        if (null != lastUltimateOfficeBasePrice.getMainPrice() && lastUltimateOfficeBasePrice.getMainPrice().compareTo(BigDecimal.ZERO) != 0) {
            BigDecimal mainPricePst =
                    inputModel.getMainPrice().divide(lastUltimateOfficeBasePrice.getMainPrice(), 4);
            officeBasePriceModifyModel.setMainPricePst(mainPricePst);
        }

        officeBasePriceModifyModel.setMainPriceRent(inputModel.getMainPriceRent_1());
        if (null != lastUltimateOfficeBasePrice.getMainPriceRent() && lastUltimateOfficeBasePrice.getMainPriceRent().compareTo(BigDecimal.ZERO) != 0) {
            BigDecimal mainPriceRentPst =
                    inputModel.getMainPriceRent().divide(lastUltimateOfficeBasePrice.getMainPriceRent(), 4);
            officeBasePriceModifyModel.setMainPriceRentPst(mainPriceRentPst);
        }

        return officeBasePriceModifyModel;
    }

    /**
     * 上期价格
     *
     * @param inputModel
     * @param lastUltimateOfficeBasePrice
     * @return
     */
    private OfficeBasePriceModifyModel compareLastYearMonth(UltimateOfficeBasePrice inputModel,
                                                            UltimateOfficeBasePrice lastUltimateOfficeBasePrice) {
        BigDecimal inputMainPrice = inputModel.getMainPrice_1();
        BigDecimal mainPrice_1 = lastUltimateOfficeBasePrice.getMainPrice();
        BigDecimal inputMainPriceRent = inputModel.getMainPriceRent_1();
        BigDecimal mainPriceRent_1 = lastUltimateOfficeBasePrice.getMainPriceRent();
        if (Objects.equals(inputMainPrice, mainPrice_1) && Objects.equals(inputMainPriceRent, mainPriceRent_1)) {
            return null;
        }

        OfficeBasePriceModifyModel officeBasePriceModifyModel =
                new OfficeBasePriceModifyModel(lastUltimateOfficeBasePrice.getId(),
                        lastUltimateOfficeBasePrice.getYearMonth());
        officeBasePriceModifyModel.setMainPrice(inputModel.getMainPrice_1());
        officeBasePriceModifyModel.setMainPricePst(lastUltimateOfficeBasePrice.getMainPricePst());
        officeBasePriceModifyModel.setMainPriceRent(inputModel.getMainPriceRent_1());
        officeBasePriceModifyModel.setMainPriceRentPst(lastUltimateOfficeBasePrice.getMainPriceRentPst());

        return officeBasePriceModifyModel;
    }

    /**
     * 计算
     */
    @Override
//    @Scheduled(cron = "")
    public void compute() {

        Date date = new Date();
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        calendar.set(calendar.get(Calendar.YEAR), calendar.get(Calendar.MONTH), 15);
        Date startDate = calendar.getTime();
        Integer lastYearMonth = new Integer(String.format("%d%02d", calendar.get(Calendar.YEAR),
                calendar.get(Calendar.MONTH) + 1));
        calendar.add(Calendar.MONTH, 1);
        Date endDate = calendar.getTime();
        Integer yearMonth = new Integer(String.format("%d%02d", calendar.get(Calendar.YEAR),
                calendar.get(Calendar.MONTH) + 1));

        before(yearMonth, startDate, endDate);
        running(yearMonth, lastYearMonth);
    }

    /**
     * 创建表
     *
     * @param yearMonth
     */
    private void before(Integer yearMonth, Date startDate, Date endDate) {
        // 创建表
        originalOfficeCaseMapper.createTable(yearMonth);
        originalOfficeCaseMapper.createArtificialTable(yearMonth);
        // 下载列表
        List<OriginalOfficeCase> downloadList = downloadOriginalOfficeCaseMapper.download(startDate, endDate);
        SqlParameterSource[] batchParams = SqlParameterSourceUtils.createBatch(downloadList.toArray());
        namedParameterJdbcTemplate.batchUpdate("insert into dbo.ODS_OFFICECASELISTED_" + yearMonth.toString() + "_RAW" +
                "(url, title,容积率,总价售,均价售, 楼盘名称, 楼盘名称_M, 楼层, 面积, 物业费, 工位数, 地址, 地铁, 发布时间, 房源编号, 百度lng, 百度lat, 区域, 分类, " +
                "来源, 等级, 楼盘网址, 装修,类型, 板块, 挂牌中介, 月租金租, 标准租金租, 得房率, 总价, 单价) values(:url,:title,:floorAreaRatio," +
                ":caseTotalPrice," +
                ":caseUnitPrice,:name,:name_m,:caseFloor,:area,:managementFee,:seatCount,:address,:metro," +
                ":publishDate,:sourceNo,:lng,:lat,:county,:catalog,:source,:level,:homePageUrl,:decoration," +
                ":type,:block,:agency,:rentOfMonthly,:rentOfStandard,:score,:totalPrice,:unitPrice) ", batchParams);
    }

    private void running(Integer yearMonth, Integer lastYearMonth) {

        String rawSql = LoadUtil.loadContent("sql-template/compute_office_price.sql");
        String sql = rawSql.replace("#yearMonth#", yearMonth.toString())
                .replace("#lastYearMonth#", lastYearMonth.toString());

        jdbcTemplate.execute(sql);
    }

    private void after() {

    }
}
